import os

FINANCIAL_INDEX_BASE_PATH = "financial_index_data"
DATA_BASE_PATH = "tickers_data"
KEY_STATS_BASE_PATH = "key_stats_data"
CLIENT_TYPES_DATA_BASE_PATH = "client_types_data"
SHAREHOLDERS_HISTORY_PATH = "shareholders_data"
pytse_dir = os.path.dirname(__file__)
LOGGER_NAME = "pytse"
