HISTORY_FIELD_MAPPINGS = {
    "<DTYYYYMMDD>": "date",
    "<FIRST>": "open",
    "<HIGH>": "high",
    "<LOW>": "low",
    "<LAST>": "close",
    "<VOL>": "volume",
    "<CLOSE>": "adjClose",
    "<OPENINT>": "count",
    "<VALUE>": "value",
    "<OPEN>": "yesterday",
}

SHAREHOLDERS_FIELD_MAPPINGS = {
    "سهامدار/دارنده": "shareholder",
    "سهم": "shares",
    "درصد": "percentage",
    "تغییر": "change",
}
